/*
 * Copyright (C) 2007 Pleyo.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Pleyo nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PLEYO AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL PLEYO OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "kjs/wk_config.h"
#include "BALConfiguration.h"
#include "BCFontPlatformDataPrivateFreeType.h"
#include <font/BTFontPlatformData.h>

#include <font/FontDescription.h>

namespace BAL {

BTFontPlatformData::BTFontPlatformData(const FontDescription& fontDescription, const AtomicString& familyName)
    : m_fontDescription(fontDescription)
    , d(0)
{
    init();
    static_cast<BCFontPlatformDataPrivateFreeType *> (d)->configureFontconfig(fontDescription, familyName);
    static_cast<BCFontPlatformDataPrivateFreeType *> (d)->configureFreetype(fontDescription);
}

BTFontPlatformData::~BTFontPlatformData()
{
    if (d)
        deleteBIFontPlatformDataPrivate(d);
    d = NULL;
}

bool BTFontPlatformData::init()
{
    d = createBIFontPlatformDataPrivate();
    ASSERT(d);
    return d->init();
}

bool BTFontPlatformData::operator==(const BTFontPlatformData& other) const
{
    if (!d && !other.d)
        return true;
    if (!d || !other.d)
            return false;
    return (d->isEqual(other.d));
}

}
