/*
 * This file is part of libtqtvd_astronav
 *
 * Copyright (C) 2008-2017 TQTVD Software
 * Copyright (C) 2017 Mirakulo Software (mirakulo@mirakulo.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */

#ifndef BCFontPlatformDataPrivateGraphicManager_h
#define BCFontPlatformDataPrivateGraphicManager_h

#include <font/BIFontPlatformDataPrivate.h>
#include <font/FontDescription.h>

#include "IGraphicManagerAdapter.h"

using WebCore::AtomicString;
using WebCore::FontDescription;
using WebCore::FontFamily;
using namespace TQTVD::GraphicManager;

namespace BAL {

class BCFontPlatformDataPrivateGraphicManager : public BIFontPlatformDataPrivate {
    public:

	BCFontPlatformDataPrivateGraphicManager();

	~BCFontPlatformDataPrivateGraphicManager();

	virtual int init();

	virtual bool isFixedPitch();

	virtual unsigned hash() const;

	virtual bool isEqual(const BIFontPlatformDataPrivate*) const;

	virtual void configureFontMapping(const FontDescription& fontDescription, const AtomicString& familyName);

	IGraphicManagerAdapterFont * getFont() const { return m_font; }

    private:
	const char * getFontFileFromDescription(const FontDescription& fontDescription);

	static int m_isInitialized;

	IGraphicManagerAdapterFont * m_font;

	IGraphicManagerAdapterFontMapping * m_fontMapping;
};

} //namespace BAL

#endif //BCFontPlatformDataPrivateGraphicManager_h
