/*
 * This file is part of libtqtvd_astronav
 *
 * Copyright (C) 2008-2017 TQTVD Software
 * Copyright (C) 2017 Mirakulo Software (mirakulo@mirakulo.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */

#include "kjs/wk_config.h"
#include "BALConfiguration.h"
#include "BCFontPlatformDataPrivateGraphicManager.h"
#include <font/BTFontPlatformData.h>

#include "BTLogHelper.h"

namespace BAL {

BTFontPlatformData::BTFontPlatformData(const FontDescription& fontDescription, const AtomicString& familyName)
    : m_fontDescription(fontDescription)
    , d(0)
{
    DBGML(MODULE_FONTS, LEVEL_INFO, "\nBTFontPlatformData::BTFontPlatformData()\n");
    init();
    static_cast<BCFontPlatformDataPrivateGraphicManager *> (d)->configureFontMapping(fontDescription, familyName);
}

BTFontPlatformData::~BTFontPlatformData()
{
    DBGML(MODULE_FONTS, LEVEL_INFO, "\nBTFontPlatformData::~BTFontPlatformData()\n");
    if (d)
        deleteBIFontPlatformDataPrivate(d);
    d = NULL;
}

bool BTFontPlatformData::init()
{
    DBGML(MODULE_FONTS, LEVEL_INFO, "\nBTFontPlatformData::init()\n");
    d = createBIFontPlatformDataPrivate();
    ASSERT(d);
    return d->init();
}

bool BTFontPlatformData::operator==(const BTFontPlatformData& other) const
{
    DBGML(MODULE_FONTS, LEVEL_INFO, "\nBTFontPlatformData::operator==\n");
    if (!d && !other.d)
            return true;
    if (!d || !other.d)
            return false;
    return (d->isEqual(other.d));
}

} //namespace BAL
