/*
 * Copyright (C) 2007 Pleyo.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Pleyo nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PLEYO AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL PLEYO OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file  BIWindowManager.h
 *
 * Header file for BIWindowManager.
 *
 * Repository informations :
 * - $URL$
 * - $Rev$
 * - $Date$
 */

#ifndef BIWINDOWMANAGER_H
#define BIWINDOWMANAGER_H

namespace WebCore {

    class IntRect;

}

namespace BAL {

    class BIEvent;
    class BIWindow;
    class BTWidget;

/**
 * The BIWindowManager implementation
 */
class BIWindowManager {
public:

    /**
     * Create and open a window.
     * @param[in] x: x position of the window
     * @param[in] y: y position of the window
     * @param[in] w: width of the window
     * @param[in] h: height of the window
     * @return BIWindow*: a pointer to the opened window
     */
    virtual BIWindow* openWindow(const uint16_t x, const uint16_t y, const uint16_t w, const uint16_t h) = 0;

    /**
     * Close a window.
     * @param[in] BIWindow*: the window to destroy
     */
    virtual void closeWindow(BIWindow*) = 0;

    /**
     * Close all windows
     * @param[in] BIWindow*: the window to destroy
     */
    virtual void closeAllWindows(void) = 0;	

    /**
     * Retrieve the active window
     * @return BIWindow*: a pointer to the active window
     */
    virtual BIWindow* activeWindow() = 0;

    /**
     * Set the active window
     * @return BIWindow*: a pointer a window
     */
    virtual void setActiveWindow(BIWindow*) = 0;

    /**
     * handle the received event if it is a window event, else send it to the frame.
     * @param[in] BIEvent*: a pointer to the event to handle
     */
    virtual void handleEvent(BIEvent*) = 0;

    virtual ~BIWindowManager() {}
};

}

#endif // BIWINDOWMANAGER_H


