export PLATFORM_TARGET
#export VERBOSE_SCRIPTS="1" // Generate verbose build for all project modules

###########################################################################################
# Environment Configuration for the platform
###########################################################################################
export LC_ALL=en_US

export PRODUCTION_FLAG="YES"
export RELEASE_VERSION="YES"

export BASE_DIR=/work
export SDK=${BASE_DIR}/sdk
export COMMON=${SDK}/common
export TOOLCHAIN=${SDK}/toolchain

export PATH:=${COMMON}/bison-2.5.1/bin:${COMMON}/bin:${TOOLCHAIN}:${PATH}

export CMAKE_EXEC=/work/sdk/common/cmake/bin/cmake

export SDK_CUSTOM_PLATFORM_ROOT_DIR=${SDK}/sony

export SDK_LINUX_NAME=SOEM_4K_linux
export SDK_PLATFORM_NAME=Unicorn_4K_BZ

export TOOLCHAIN=${SDK}/toolchain
export FONT_PREFIX=
export CROSSHOST=armv7a-mediatek482_001_neon-linux-gnueabi

export PREFIX=${TOOLCHAIN}/${CROSSHOST}

export AR=${CROSSHOST}-ar
export AS=${CROSSHOST}-as
export LD=${CROSSHOST}-ld
export NM=${CROSSHOST}-nm
export CC=${CROSSHOST}-gcc
export CPP=${CROSSHOST}-cpp
export GCC=${CROSSHOST}-gcc
export CXX=${CROSSHOST}-g++
export OBJCOPY=${CROSSHOST}-objcopy
export RANLIB=${CROSSHOST}-ranlib

export cc=${CC}
export cpp=${CPP}
export cxx=${CXX}
export as=${AS}
export ar=${AR}
export ld=${LD}
export ranlib=${RANLIB}

export EXTERNALSROOT=${BASE_DIR}/externals
export SDKROOT=${BASE_DIR}/sdk
export CONFIGSROOT=${BASE_DIR}/configs
export DEPLOY_DIR=${BASE_DIR}/deploy
export BUILD_DIR=${BASE_DIR}/build

## Revision variable to be used by any product component
export PROJECT_REVISION="100"
export REVISION_SDK="100"


export TQTVD_BUILD_ASTRONAV=SHARED

###########################################################################################
## Platform attributes
export MAX_MEM_GM_VID=25165824
export MAX_MEM_GM_SYS=0

###########################################################################################
## Build flags

ifeq ($(findstring debug, $(MAKECMDGOALS)), debug)
	COREFLAGS=-O0 -ggdb -fno-omit-frame-pointer -rdynamic \
	-funwind-tables # Required to obtain stack traces on some platforms
	export STRIP=null.sh
else
	COREFLAGS=-O2 -march=armv7-a -mtune=cortex-a9 -mfloat-abi=softfp -mfpu=neon-vfpv4 -DNDEBUG
	
	ifeq ($(findstring nostrip, $(MAKECMDGOALS)), nostrip)
		COREFLAGS += -fno-omit-frame-pointer -rdynamic \
	-funwind-tables # Required to obtain stack traces on some platforms
		export STRIP=null.sh
	else
		COREFLAGS += -fomit-frame-pointer
		export STRIP=${CROSSHOST}-strip
	endif
endif

COREFLAGS += -DMODEL=FF # to be defined

BASEFLAGS=-fPIC -Wall -Wno-undef -Wno-cast-qual -Wno-comment -Wno-write-strings -Wno-parentheses -Wreturn-type \
-Wunused-value -Wunused-variable -Wpointer-arith -Wno-unused-function -Wno-unused-but-set-variable -Wno-deprecated-declarations \
-fno-strict-aliasing -fsigned-char -fno-aggressive-loop-optimizations -fno-builtin -ffunction-sections \
-fdata-sections -fno-short-enums \
-DLINUX_THREADS=8000 -DNPTL=8001 -D_LINUX_ -D__ARM__ -D__arm__ -D_REENTRANT \
-DOPENSSL_NO_DGRAM -DNO_SYSLOG -DNO_SYS_UN_H \
-D_GNU_SOURCE -D__ARM_EABI__ -DFAR=

BASEFLAGS += -I${SDK}/sony/apollo/oss/library/gnuarm-4.8.2_neon_ca9/zlib/1.2.3/pre-install/include \
-I${SDK}/sony/apollo/oss/library/gnuarm-4.8.2_neon_ca9/mng/1.0.10/pre-install/include \
-I${SDK}/sony/apollo/linux_core/driver/directfb/directfb_1.5.3/Source/DirectFB/include \
-I${SDK}/sony/apollo/linux_core/driver/directfb/directfb_1.5.3/Source/DirectFB/lib \
-I${SDK}/sony/apollo/mtk_obj/SOEM_4K_linux/Unicorn_4K_BZ/rel/inc/c_inc \
-I${SDK}/sony/apollo/middleware/public


CFLAGS=${COREFLAGS} ${BASEFLAGS} -Wstrict-prototypes
CFLAGS_DBG=${COREFLAGS_DBG} ${BASEFLAGS} -Wstrict-prototypes

CXXFLAGS=${COREFLAGS} ${BASEFLAGS}
CXXFLAGS_DBG=${COREFLAGS_DBG} ${BASEFLAGS}

LDFLAGS = \
-L${SDK}/sony/apollo/sys_build/SOEM_4K_linux/Unicorn_4K_BZ \
-L${SDK}/target/usr/lib -lpthread \
-L${SDK}/target/lib -lz

export CFLAGS CFLAGS_DBG CXXFLAGS CXXFLAGS_DBG LDFLAGS
